from django.contrib import admin
from . import models



@admin.register(models.Service)
class ServiceAdmin(admin.ModelAdmin):
    fields = ("name",
              "clients",
              "comments",
              "backup_profile",
    )

@admin.register(models.Application)
class ApplicationAdmin(admin.ModelAdmin):
    fields = ("name",
              "protocol",
              "ports",
              "version",
              "devices",
              "vm",
    )

@admin.register(models.IC)
class ICAdmin(admin.ModelAdmin):
    fields = ("service",
              "assigned_object_type",
              "assigned_object_id",)

@admin.register(models.Relation)
class RelationAdmin(admin.ModelAdmin):
    fields = ("service",
              "source",
              "source_shape",
              "destination",
              "destination_shape",
              "connector_shape",
    )

@admin.register(models.PenTest)
class PenTestAdmin(admin.ModelAdmin):
    fields = ("service",
              "status",
              "date",
              "ticket",
              "report_link")




"""
from rest_framework.serializers import ModelSerializer

from nb_service.models import MyModel1


class MyModel1Serializer(ModelSerializer):

    class Meta:
        model = MyModel1
        fields = '__all__'
"""
from drf_yasg.utils import swagger_serializer_method
from rest_framework import serializers
from django.contrib.contenttypes.models import ContentType
from netbox.api import ChoiceField, ContentTypeField, WritableNestedSerializer
from utilities.api import get_serializer_for_model
from tenancy.api.nested_serializers import NestedTenantSerializer
from ipam.choices import ServiceProtocolChoices
from dcim.api.nested_serializers import NestedDeviceSerializer
from virtualization.api.nested_serializers import NestedVirtualMachineSerializer

from nb_service import models
from nb_service import choices

class NestedApplicationSerializer(WritableNestedSerializer):
    id = serializers.IntegerField(read_only=True)
    name = serializers.CharField()

    class Meta:
        model = models.Application
        fields = ['id', 'name',]


class nb_Applicationserializer(serializers.Serializer):
    id = serializers.IntegerField(read_only=True)
    name = serializers.CharField()
    display = serializers.SerializerMethodField('get_display')
    protocol = ChoiceField(choices=ServiceProtocolChoices, required=False)
    version = serializers.CharField()
    devices = NestedDeviceSerializer(many=True,required=False, allow_null=True)
    vm = NestedVirtualMachineSerializer(many=True,required=False, allow_null=True)

    def get_display(self, obj):
        return f"{obj}"

    class Meta:
        model = models.Application
        fields = [
            'id',
            'display',
            'name',
            'protocol',
            'ports',
            'version',
            'devices',
            'vm',
            ]

class nb_ICserializer(serializers.Serializer):

    name = serializers.CharField(read_only=True)
    display = serializers.SerializerMethodField('get_display')
    service = serializers.CharField(source='service.name', required=False)

    id = serializers.IntegerField(read_only=True)
    service_id = serializers.IntegerField(source='service.id', required=True,)

    assigned_object_type = ContentTypeField(
        queryset=ContentType.objects.filter(choices.OBJETO_ASSIGNMENT_MODELS),
        required=True,
        allow_null=True
    )
    assigned_object = serializers.SerializerMethodField(read_only=True)

    assigned_object_id = serializers.IntegerField(source='assigned_object.id')
    
    def get_display(self, obj):
        return obj.name

    @swagger_serializer_method(serializer_or_field=serializers.DictField)
    def get_assigned_object(self, obj):
        if obj.assigned_object is None:
            return None
        serializer = get_serializer_for_model(obj.assigned_object, prefix='Nested')
        context = {'request': self.context['request']}
        return serializer(obj.assigned_object, context=context).data


    class Meta:
        model = models.IC
        fields = ['id', 'display', 'name', 'service', 'assigned_object_type', 'assigned_object', 'assigned_object_id']

class nb_ServiceSerializer(serializers.Serializer):

    name = serializers.CharField()
    display = serializers.SerializerMethodField('get_display')
    clients = NestedTenantSerializer(many=True,required=False, allow_null=True)
    comments = serializers.CharField()
    backup_profile = serializers.CharField(required=False)


    def get_display(self, obj):
        return obj.name

    class Meta:
        model = models.Service
        fields = ['id', 'display', 'name', 'clients', 'comments', 'backup_profile']


from django import forms

from dcim.models import Device
from circuits.models import Circuit, Provider
from tenancy.models import Tenant
from ipam.models import Service
from virtualization.models import VirtualMachine
from utilities.forms import BootstrapMixin, DatePicker, DynamicModelMultipleChoiceField, DynamicModelChoiceField


from . import utils
from . import models

class ServiceForm(BootstrapMixin, forms.ModelForm):

    clients = DynamicModelMultipleChoiceField(label="Clients",
        queryset=Tenant.objects.all(),
        required=True,
    )

    backup_profile = forms.CharField(required=False)

    class Meta:
        model = models.Service
        fields = [
            "name",
            "clients",
            "comments",
            "backup_profile",
        ]

class ApplicationForm(BootstrapMixin, forms.ModelForm):

    devices = DynamicModelMultipleChoiceField(label="Devices",
        queryset=Device.objects.all(),
        required=False,
    )
    vm = DynamicModelMultipleChoiceField(label="Virtual Machines",
        queryset=VirtualMachine.objects.all(),
        required=False,
    )


    class Meta:
        model = models.Application
        fields = [
            "name",
            "protocol",
            "ports",
            "version",
            "devices",
            "vm",
        ]

class ICForm(BootstrapMixin, forms.ModelForm):

    device = DynamicModelChoiceField(
        queryset=Device.objects.all(),
        required=False,
    )
    virtual_machine = DynamicModelChoiceField(
        queryset=VirtualMachine.objects.all(),
        required=False,
    )

    application = utils.DynamicModelChoiceField(
        queryset=models.Application.objects.all(),
        required=False,
    )

    class Meta:
        model = models.IC
        fields = [
            'service',
        ]


class PenTestForm(BootstrapMixin, forms.ModelForm):
    
    class Meta:
        model = models.PenTest
        fields = [
            'service',
            'comments',
            'status',
            'date',
            'ticket',
            "report_link",
        ]

        widgets = {
            'date': DatePicker(),
        }

class RelationForm(BootstrapMixin, forms.ModelForm):

    source = utils.DynamicModelChoiceField(
        queryset=models.IC.objects.all(),
        required=True,
        label='Source',
        query_params={
            'service_id': '$service',
        },
        display_field='name',
    )

    destination = utils.DynamicModelChoiceField(
        queryset=models.IC.objects.all(),
        required=True,
        label='Destination',
        query_params={
            'service_id': '$service',
        },
        display_field='name',
    )

    link_text = forms.CharField(required=False)

    class Meta:
        model = models.Relation
        fields = [
            'service',
            'source',
            'source_shape',
            'destination',
            'destination_shape',
            "connector_shape",
            "link_text",
        ]

class ServiceFilterForm(BootstrapMixin, forms.ModelForm):
    
    q = forms.CharField(
        required=False,
        label='Search'
    )
    clients = DynamicModelMultipleChoiceField(label="Clients",
        queryset=Tenant.objects.all(),
        required=False,
    )

    class Meta:
        model = models.Service 
        fields = [ 
            'q',
            'clients',
        ]

class ApplicationFilterForm(BootstrapMixin, forms.ModelForm):
    
    q = forms.CharField(
        required=False,
        label='Search'
    )
    devices = DynamicModelMultipleChoiceField(label="Devices",
        queryset=Device.objects.all(),
        required=False,
    )
    
    virtual_machines = DynamicModelMultipleChoiceField(label="Virtual Machines",
        queryset=VirtualMachine.objects.all(),
        required=False,
    )

    class Meta:
        model = models.Service 
        fields = [ 
            'q',
            'devices',
            'virtual_machines',
        ]



import logging
from . import models 
from . import filters
from . import forms 
from . import tables
from django.conf import settings
from packaging import version
from django.contrib.contenttypes.models import ContentType
from netbox.views import generic
from tenancy.tables import TenantTable
from virtualization.models import VirtualMachine
from dcim.models import Device
from dcim.tables import DeviceTable
from virtualization.tables import VirtualMachineTable
from utilities.utils import normalize_querydict 
from utilities.forms import restrict_form_fields, ConfirmationForm
from django.shortcuts import get_object_or_404, redirect, render
from django.db import transaction
from django.utils.html import escape
from django.contrib import messages
from django.utils.safestring import mark_safe
from utilities.permissions import get_permission_for_model



NETBOX_CURRENT_VERSION = version.parse(settings.VERSION)


class ServiceListView(generic.ObjectListView):
    queryset = models.Service.objects.all()
    table = tables.ServiceTable
    filterset = filters.ServiceFilter
    filterset_form = forms.ServiceFilterForm
    action_buttons = ("add")
    actions = action_buttons

    def __init__(self, *args, **kwargs):
        if NETBOX_CURRENT_VERSION >= version.parse("3.0") :
            self.template_name = 'nb_service/3.x/custom_list.html'
        else:
            self.template_name = 'nb_service/2.x/custom_list.html'        
        super().__init__(*args, *kwargs)

    def get_extra_context(self, request):
        data = {}
        model = self.queryset.model
        permissions = {}
        for action in ('add', 'change', 'delete', 'view'):
            perm_name = get_permission_for_model(model, action)
            permissions[action] = request.user.has_perm(perm_name)

        content_type = ContentType.objects.get_for_model(model)
        if NETBOX_CURRENT_VERSION >= version.parse("3.2"):
            data['content_type'] = content_type
            data['permissions'] = permissions
            data['action_buttons'] = self.action_buttons

        return data

class ServiceView(generic.ObjectView):
    queryset = models.Service.objects.all()

    def __init__(self, *args, **kwargs):
        if NETBOX_CURRENT_VERSION >= version.parse("3.0") :
            self.template_name = 'nb_service/3.x/service/service_view.html'
        else:
            self.template_name = 'nb_service/2.x/service/service_view.html'     
        super().__init__(*args, *kwargs)

    def get_extra_context(self, request, instance):
        items_obj = instance.config_itens.all()
        objetos_table = tables.ICTable(items_obj)
        tenants_table = TenantTable(instance.clients.all())
        vuln_table = tables.VulnTable(instance.pentest_reports.all())
        model = self.queryset.model
        permissions = {}
        for action in ('add', 'change', 'delete', 'view'):
            perm_name = get_permission_for_model(model, action)
            permissions[action] = request.user.has_perm(perm_name)

        data = {
                "tenant_table" : tenants_table,
                "permissions": permissions,
                "vuln_table" : vuln_table,
                "objetos_table": objetos_table,
                "config_items" : items_obj,
            }
        return data


class ServiceDiagramView(generic.ObjectView):
    queryset = models.Service.objects.all()

    def __init__(self, *args, **kwargs):
        if NETBOX_CURRENT_VERSION >= version.parse("3.0") :
            self.template_name = 'nb_service/3.x/service/service_diagram_view.html'
        else:
            self.template_name = 'nb_service/2.x/service/service_diagram_view.html'  
        super().__init__(*args, *kwargs)

class ServiceICView(generic.ObjectView):
    queryset = models.Service.objects.all()

    def __init__(self, *args, **kwargs):
        if NETBOX_CURRENT_VERSION >= version.parse("3.0") :
            self.template_name = 'nb_service/3.x/service/service_IC_view.html'
        else:
            self.template_name = 'nb_service/2.x/service/service_IC_view.html'
        super().__init__(*args, *kwargs)

    def get_extra_context(self, request, instance):
        items_obj = instance.config_itens.all()
        data = {
                "config_items" : items_obj,
            }
        return data

class ServiceRelationView(generic.ObjectView):
    queryset = models.Service.objects.all()

    def __init__(self, *args, **kwargs):
        if NETBOX_CURRENT_VERSION >= version.parse("3.0") :
            self.template_name = 'nb_service/3.x/service/service_Relation_view.html'
        else:
            self.template_name = 'nb_service/2.x/service/service_Relation_view.html'
        super().__init__(*args, *kwargs)

    def get_extra_context(self, request, instance):
        relations = instance.relationships.all()
        data = {
                "relations_objs" : relations,
            }
        return data


class ServiceEditView(generic.ObjectEditView):
    queryset = models.Service.objects.all()
    model_form = forms.ServiceForm
    form  = forms.ServiceForm

    def __init__(self, *args, **kwargs):
        if NETBOX_CURRENT_VERSION >= version.parse("3.0") :
            self.template_name = 'nb_service/3.x/obj_edit.html'
        else:
            self.template_name = 'nb_service/2.x/obj_edit.html'
        super().__init__(*args, *kwargs)
    
    def get_extra_context(self, request, instance):
        data = {}
        if NETBOX_CURRENT_VERSION >= version.parse("3.2"):
            data['obj'] = instance

        return data

if NETBOX_CURRENT_VERSION >= version.parse("3.0") :
    from nb_service.views_3_x import ServiceDeleteView as ServiceDeleteView_3x
    from nb_service.views_3_x import ApplicationDeleteView as ApplicationDeleteView_3x
    ServiceDeleteView = ServiceDeleteView_3x
    ApplicationDeleteView = ApplicationDeleteView_3x
else:
    from nb_service.views_2_x import ServiceDeleteView as ServiceDeleteView_2x
    from nb_service.views_2_x import ApplicationDeleteView as ApplicationDeleteView_2x
    ServiceDeleteView = ServiceDeleteView_2x
    ApplicationDeleteView = ApplicationDeleteView_2x


class ICCreateView(generic.ObjectEditView):
    queryset = models.IC.objects.all()
    model_form = forms.ICForm
    form = forms.ICForm

    def __init__(self, *args, **kwargs):
        if NETBOX_CURRENT_VERSION >= version.parse("3.0") :
            self.template_name = 'nb_service/3.x/IC/ic_edit.html'
        else:
            self.template_name = 'nb_service/2.x/IC/ic_edit.html'
        super().__init__(*args, *kwargs)
        self.alter_object = self.alter_obj

    def get_extra_context(self, request, instance):
        data = {}
        if NETBOX_CURRENT_VERSION >= version.parse("3.2"):
            data['obj'] = instance

        return data

    def alter_obj(self, obj, request, url_args, url_kwargs):
        if 'device' in request.POST:
            try:
                obj.assigned_object =  Device.objects.get(pk=request.POST['device'])
            except (ValueError, Device.DoesNotExist):
                pass

        if 'virtual_machine' in request.POST:
            try:
                obj.assigned_object =  VirtualMachine.objects.get(pk=request.POST['virtual_machine'])
            except (ValueError, Device.DoesNotExist):
                pass
        if 'application' in request.POST:
            try:
                obj.assigned_object =  models.Application.objects.get(pk=request.POST['application'])
            except (ValueError, Device.DoesNotExist):
                pass
        

        return obj


class RelationEditView(generic.ObjectEditView):
    queryset = models.Relation.objects.all()
    model_form = forms.RelationForm
    form = forms.RelationForm

    def __init__(self, *args, **kwargs):
        if NETBOX_CURRENT_VERSION >= version.parse("3.0") :
            self.template_name = 'nb_service/3.x/obj_edit.html'
        else:
            self.template_name = 'nb_service/2.x/obj_edit.html'
        super().__init__(*args, *kwargs)
    
    def get_extra_context(self, request, instance):
        data = {}
        if NETBOX_CURRENT_VERSION >= version.parse("3.2"):
            data['obj'] = instance

        return data

class PenTestEditView(generic.ObjectEditView):
    queryset = models.PenTest.objects.all()
    model_form = forms.PenTestForm
    form = forms.PenTestForm

    def __init__(self, *args, **kwargs):
        if NETBOX_CURRENT_VERSION >= version.parse("3.0") :
            self.template_name = 'nb_service/3.x/obj_edit.html'
        else:
            self.template_name = 'nb_service/2.x/obj_edit.html'
        super().__init__(*args, *kwargs)

    def get_extra_context(self, request, instance):
        data = {}
        if NETBOX_CURRENT_VERSION >= version.parse("3.2"):
            data['obj'] = instance

        return data

class PenTestDeleteView(generic.ObjectDeleteView):
    queryset = models.PenTest.objects.all()

class RelationDeleteView(generic.ObjectDeleteView):
    queryset = models.Relation.objects.all()

class ICDeleteView(generic.ObjectDeleteView):
    queryset = models.IC.objects.all()


class ApplicationListView(generic.ObjectListView):
    queryset = models.Application.objects.all()
    table = tables.ApplicationTable
    filterset = filters.ApplicationFilter
    filterset_form = forms.ApplicationFilterForm
    action_buttons = ("add")

    def __init__(self, *args, **kwargs):
        if NETBOX_CURRENT_VERSION >= version.parse("3.0") :
            self.template_name = 'nb_service/3.x/custom_list.html'
        else:
            self.template_name = 'nb_service/2.x/custom_list.html'
        super().__init__(*args, *kwargs)

    def get_extra_context(self, request):
        data = {}
        model = self.queryset.model
        permissions = {}
        for action in ('add', 'change', 'delete', 'view'):
            perm_name = get_permission_for_model(model, action)
            permissions[action] = request.user.has_perm(perm_name)

        content_type = ContentType.objects.get_for_model(model)
        if NETBOX_CURRENT_VERSION >= version.parse("3.2"):
            data['content_type'] = content_type
            data['permissions'] = permissions
            data['action_buttons'] = self.action_buttons

        return data

class ApplicationView(generic.ObjectView):
    queryset = models.Application.objects.all()

    def __init__(self, *args, **kwargs):
        if NETBOX_CURRENT_VERSION >= version.parse("3.0") :
            self.template_name = 'nb_service/3.x/Application/application_view.html'
        else:
            self.template_name = 'nb_service/2.x/Application/application_view.html'
        super().__init__(*args, *kwargs)

class ApplicationDevicesView(generic.ObjectView):
    queryset = models.Application.objects.all()

    def __init__(self, *args, **kwargs):
        if NETBOX_CURRENT_VERSION >= version.parse("3.0") :
            self.template_name = 'nb_service/3.x/Application/application_objs.html'
        else:
            self.template_name = 'nb_service/2.x/Application/application_objs.html'
        super().__init__(*args, *kwargs)

    def get_extra_context(self, request, instance):
        objs = instance.devices.all()
        table = DeviceTable(objs)
        return {
            'Object_type': 'Devices',
            'active_tab' : 'device',
            'table': table,
        }

class ApplicationVMsView(generic.ObjectView):
    queryset = models.Application.objects.all()

    def __init__(self, *args, **kwargs):
        if NETBOX_CURRENT_VERSION >= version.parse("3.0") :
            self.template_name = 'nb_service/3.x/Application/application_objs.html'
        else:
            self.template_name = 'nb_service/2.x/Application/application_objs.html'
        super().__init__(*args, *kwargs)


    def get_extra_context(self, request, instance):
        objs = instance.vm.all()
        table = VirtualMachineTable(objs)
        return {
            'Object_type': 'Virtual Machines',
            'active_tab' : 'virtual_machine',
            'table': table,
        }

class ApplicationEditView(generic.ObjectEditView):
    queryset = models.Application.objects.all()
    model_form = forms.ApplicationForm
    form = forms.ApplicationForm

    def __init__(self, *args, **kwargs):
        if NETBOX_CURRENT_VERSION >= version.parse("3.0") :
            self.template_name = 'nb_service/3.x/obj_edit.html'
        else:
            self.template_name = 'nb_service/2.x/obj_edit.html'
        super().__init__(*args, *kwargs)

    def get_extra_context(self, request, instance):
        data = {}
        if NETBOX_CURRENT_VERSION >= version.parse("3.2"):
            data['obj'] = instance

        return data
    



from . import models 
from netbox.views import generic
from django.conf import settings
from packaging import version

from utilities.htmx import is_htmx
from utilities.forms import ConfirmationForm
from django.urls import reverse
from django.shortcuts import get_object_or_404, redirect, render

NETBOX_CURRENT_VERSION = version.parse(settings.VERSION)
if NETBOX_CURRENT_VERSION >= version.parse("3.2"):
    from netbox.views.generic import ObjectChangeLogView
else:
    from extras.views import ObjectChangeLogView



class ServiceDeleteView(generic.ObjectDeleteView):
    queryset = models.Service.objects.all()

    def get(self, request, **kwargs):
        if NETBOX_CURRENT_VERSION >= version.parse("3.2"):
            obj = self.get_object(**kwargs)
        else:
            obj = self.get_object(kwargs)
        
        form = ConfirmationForm(initial=request.GET)

        # If this is an HTMX request, return only the rendered deletion form as modal content
        if is_htmx(request):
            viewname = f'plugins:{self.queryset.model._meta.app_label}:{self.queryset.model._meta.model_name}_delete'
            form_url = reverse(viewname, kwargs={'pk': obj.pk})
            return render(request, 'htmx/delete_form.html', {
                'object': obj,
                'object_type': self.queryset.model._meta.verbose_name,
                'form': form,
                'form_url': form_url,
            })

        return render(request, self.template_name, {
            'object': obj,
            'object_type': self.queryset.model._meta.verbose_name,
            'form': form,
            'return_url': self.get_return_url(request, obj),
        })


class ApplicationDeleteView(generic.ObjectDeleteView):
    queryset = models.Application.objects.all()

    def get(self, request, **kwargs):
        if NETBOX_CURRENT_VERSION >= version.parse("3.2"):
            obj = self.get_object(**kwargs)
        else:
            obj = self.get_object(kwargs)
        
        form = ConfirmationForm(initial=request.GET)

        # If this is an HTMX request, return only the rendered deletion form as modal content
        if is_htmx(request):
            viewname = f'plugins:{self.queryset.model._meta.app_label}:{self.queryset.model._meta.model_name}_delete'
            form_url = reverse(viewname, kwargs={'pk': obj.pk})
            return render(request, 'htmx/delete_form.html', {
                'object': obj,
                'object_type': self.queryset.model._meta.verbose_name,
                'form': form,
                'form_url': form_url,
            })

        return render(request, self.template_name, {
            'object': obj,
            'object_type': self.queryset.model._meta.verbose_name,
            'form': form,
            'return_url': self.get_return_url(request, obj),
        })


class ServiceChangeLogView(ObjectChangeLogView):
    base_template = 'nb_service/3.x/service/service_ChangeLog_view.html'

class ApplicationChangeLogView(ObjectChangeLogView):
    base_template = 'nb_service/3.x/Application/application_ChangeLog.html'


from django.conf import settings
from packaging import version
import django_tables2 as tables


NETBOX_CURRENT_VERSION = version.parse(settings.VERSION)
if NETBOX_CURRENT_VERSION >= version.parse("3.2") :
    from netbox.tables import NetBoxTable, ToggleColumn
else: 
    from utilities.tables import BaseTable as NetBoxTable
    from utilities.tables import ToggleColumn

from . import models


class ServiceTable(NetBoxTable):

    name = tables.LinkColumn(verbose_name="Service")
    id = ToggleColumn()
    backup_profile = tables.Column(verbose_name="Backup Profile")

    
    class Meta(NetBoxTable.Meta):
        model = models.Service
        fields = (
            "name",
            "backup_profile",
        )

class ICTable(NetBoxTable):
    id = ToggleColumn()
    assigned_object = tables.LinkColumn(verbose_name="CI")
    obj_type = tables.Column(verbose_name="Type")
    
    class Meta(NetBoxTable.Meta):
        model = models.IC        
        fields = (
            "id",
            "assigned_object",
            "obj_type"
        )

class VulnTable(NetBoxTable):
    id = ToggleColumn()

    class Meta(NetBoxTable.Meta):
        model = models.PenTest        
        fields = (
            "id",
            "ticket",
            "date",
            "status",
        )

class ApplicationTable(NetBoxTable):

    name = tables.LinkColumn(verbose_name="Application")
    id = ToggleColumn()

    class Meta(NetBoxTable.Meta):
        model = models.Application
        fields = [
            "name",
            "version",
            "protocol",
            "ports",
         ]


from extras.plugins import PluginConfig
from .version import __version__


class NbserviceConfig(PluginConfig):
    name = 'nb_service'
    verbose_name = 'Service Management'
    description = ''
    version = __version__
    author = ''
    author_email = ''
    required_settings = []
    default_settings = {}

    def ready(self):
        from . import signals
        super().ready()


config = NbserviceConfig # noqa



# Generated by Django 3.1.7 on 2022-04-22 20:51

import django.contrib.postgres.fields
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('dcim', '0122_standardize_name_length'),
        ('tenancy', '0011_standardize_name_length'),
        ('virtualization', '0019_standardize_name_length'),
    ]

    operations = [
        migrations.CreateModel(
            name='IC',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('assigned_object_id', models.PositiveIntegerField(blank=True, null=True)),
                ('assigned_object_type', models.ForeignKey(blank=True, limit_choices_to=models.Q(models.Q(models.Q(('app_label', 'dcim'), ('model', 'device')), models.Q(('app_label', 'circuits'), ('model', 'circuit')), models.Q(('app_label', 'virtualization'), ('model', 'virtualmachine')), models.Q(('app_label', 'nb_service'), ('model', 'application')), _connector='OR')), null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='contenttypes.contenttype')),
            ],
        ),
        migrations.CreateModel(
            name='Service',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('name', models.CharField(max_length=100)),
                ('comments', models.TextField(blank=True)),
                ('backup_profile', models.CharField(max_length=100)),
                ('clients', models.ManyToManyField(related_name='itsm_services', to='tenancy.Tenant')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Relation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('source_shape', models.IntegerField()),
                ('destination_shape', models.IntegerField()),
                ('connector_shape', models.IntegerField()),
                ('link_text', models.CharField(max_length=100)),
                ('destination', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='nb_service.ic')),
                ('service', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='relationships', to='nb_service.service')),
                ('source', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='nb_service.ic')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='PenTest',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('comments', models.TextField(blank=True)),
                ('status', models.IntegerField(blank=True)),
                ('date', models.DateField(blank=True)),
                ('ticket', models.CharField(max_length=100)),
                ('report_link', models.CharField(max_length=100)),
                ('service', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='pentest_reports', to='nb_service.service')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='ic',
            name='service',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='config_itens', to='nb_service.service'),
        ),
        migrations.CreateModel(
            name='Application',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('name', models.CharField(max_length=100)),
                ('protocol', models.CharField(max_length=50)),
                ('ports', django.contrib.postgres.fields.ArrayField(base_field=models.PositiveIntegerField(validators=[django.core.validators.MinValueValidator(1), django.core.validators.MaxValueValidator(65535)]), size=None)),
                ('version', models.CharField(max_length=100)),
                ('devices', models.ManyToManyField(blank=True, related_name='uses_apps', to='dcim.Device')),
                ('vm', models.ManyToManyField(blank=True, related_name='uses_apps', to='virtualization.VirtualMachine')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AlterUniqueTogether(
            name='ic',
            unique_together={('service', 'assigned_object_type', 'assigned_object_id')},
        ),
    ]



from django import template
from django.urls import NoReverseMatch, reverse

register = template.Library()

@register.filter()
def plugin_validated_viewname(model, action):
    """
    Return the view name for the given model and action if valid, or None if invalid.
    """
    viewname = f'plugins:{model._meta.app_label}:{model._meta.model_name}_{action}'
    try:
        # Validate and return the view name. We don't return the actual URL yet because many of the templates
        # are written to pass a name to {% url %}.
        reverse(viewname)
        return viewname
    except NoReverseMatch:
        return None


@register.filter()
def plugin_viewname(model, action):
    """
    Return the view name for the given model and action. Does not perform any validation.
    """
    return f'plugins:{model._meta.app_label}:{model._meta.model_name}_{action}'


from django import template
from django.urls import reverse

from extras.models import ExportTemplate
from utilities.utils import prepare_cloned_fields

register = template.Library()


def _get_viewname(instance, action):
    """
    Return the appropriate viewname for adding, editing, or deleting an instance.
    """

    # Validate action
    assert action in ('add', 'edit', 'delete')
    viewname = "plugins:{}:{}_{}".format(
        instance._meta.app_label, instance._meta.model_name, action
    )

    return viewname


#
# Instance buttons
#

@register.inclusion_tag('buttons/clone.html')
def clone_button(instance):
    url = reverse(_get_viewname(instance, 'add'))

    # Populate cloned field values
    param_string = prepare_cloned_fields(instance).urlencode()
    if param_string:
        url = f'{url}?{param_string}'

    return {
        'url': url,
    }


@register.inclusion_tag('buttons/edit.html')
def edit_button(instance):
    viewname = _get_viewname(instance, 'edit')
    url = reverse(viewname, kwargs={'pk': instance.pk})

    return {
        'url': url,
    }


@register.inclusion_tag('buttons/delete.html')
def delete_button(instance):
    viewname = _get_viewname(instance, 'delete')
    url = reverse(viewname, kwargs={'pk': instance.pk})

    return {
        'url': url,
    }


#
# List buttons
#

@register.inclusion_tag('buttons/add.html')
def add_button(url):

    url = reverse(url)

    return {
        'add_url': url,
        'url' : url
    }


@register.inclusion_tag('buttons/import.html')
def import_button(url):

    return {
        'import_url': url,
    }


@register.inclusion_tag('buttons/export.html', takes_context=True)
def export_button(context, content_type):
    user = context['request'].user

    # Determine if the "all data" export returns CSV or YAML
    data_format = 'YAML' if hasattr(content_type.model_class(), 'to_yaml') else 'CSV'

    # Retrieve all export templates for this model
    export_templates = ExportTemplate.objects.restrict(user, 'view').filter(content_type=content_type)

    return {
        'perms': context['perms'],
        'content_type': content_type,
        'url_params': context['request'].GET.urlencode() if context['request'].GET else '',
        'export_templates': export_templates,
        'data_format': data_format,
    }


from rest_framework import viewsets

from nb_service import models
from nb_service import filters
from . import serializers

class ICViewSet(viewsets.ReadOnlyModelViewSet):
    queryset = models.IC.objects.all()
    serializer_class = serializers.nb_ICserializer
    filterset_class = filters.ICFilter


class ServiceViewSet(viewsets.ReadOnlyModelViewSet):
    queryset = models.Service.objects.all()
    serializer_class = serializers.nb_ServiceSerializer

class ApplicationViewSet(viewsets.ReadOnlyModelViewSet):
    queryset = models.Application.objects.all()
    serializer_class = serializers.nb_Applicationserializer
